import React from "react";
import { createContext, useContext } from "react";
import { startOfMonth } from "date-fns/startOfMonth";
import { getDates } from "../helpers/getDates";
import { getDays } from "../helpers/getDays";
import { getDisplayMonths } from "../helpers/getDisplayMonths";
import { getDropdownMonths } from "../helpers/getDropdownMonths";
import { getDropdownYears } from "../helpers/getDropdownYears";
import { getMonths } from "../helpers/getMonths";
import { getNextMonth } from "../helpers/getNextMonth";
import { getPreviousMonth } from "../helpers/getPreviousMonth";
import { getStartMonth } from "../helpers/getStartMonth";
import { getWeeks } from "../helpers/getWeeks";
import { useControlledValue } from "../helpers/useControlledValue";
import { useProps } from "./props";
const calendarContext = createContext(undefined);
/** @private */
export function CalendarProvider(providerProps) {
    const props = useProps();
    const startMonth = getStartMonth(props);
    // The first month displayed in the calendar
    const [firstMonth, setFirstMonth] = useControlledValue(startMonth, props.month ? startOfMonth(props.month) : undefined);
    /** An array of the months displayed in the calendar. */
    const displayMonths = getDisplayMonths(firstMonth, props);
    /** The last month displayed in the calendar. */
    const lastMonth = displayMonths[displayMonths.length - 1];
    /** An array of the dates displayed in the calendar. */
    const dates = getDates(displayMonths, props.toDate, props);
    /** An array of the Months displayed in the calendar. */
    const months = getMonths(displayMonths, dates, props);
    /** An array of the Weeks displayed in the calendar. */
    const weeks = getWeeks(months);
    /** An array of the Days displayed in the calendar. */
    const days = getDays(months);
    const nextMonth = getNextMonth(firstMonth, props);
    const previousMonth = getPreviousMonth(firstMonth, props);
    function isDayDisplayed(day) {
        return weeks.some((week) => {
            return week.days.some((d) => {
                return d.isEqualTo(day);
            });
        });
    }
    function goToMonth(date) {
        if (props.disableNavigation) {
            return;
        }
        const month = startOfMonth(date);
        setFirstMonth(month);
        props.onMonthChange?.(month);
    }
    function goToDay(day) {
        if (isDayDisplayed(day)) {
            return;
        }
        // TODO:
        // if (refDate && isBefore(date, refDate)) {
        //   console.log('date is before refDate');
        //   const month = addMonths(date, 1 + dayPicker.numberOfMonths * -1);
        //   console.log('going to month', month);
        //   goToMonth(month);
        // } else {
        //   console.log('going to month', date);
        goToMonth(day.date);
        // }
    }
    function goToNextMonth() {
        return nextMonth ? goToMonth(nextMonth) : undefined;
    }
    function goToPreviousMonth() {
        return previousMonth ? goToMonth(previousMonth) : undefined;
    }
    const isInteractive = props.mode !== "default" || props.onDayClick !== undefined;
    const calendar = {
        dates,
        months,
        weeks,
        days,
        firstMonth,
        lastMonth,
        previousMonth,
        nextMonth,
        goToMonth,
        goToNextMonth,
        goToPreviousMonth,
        goToDay,
        isInteractive,
        isDayDisplayed,
        dropdownOptions: {
            months: getDropdownMonths(props),
            years: getDropdownYears(props)
        }
    };
    return (React.createElement(calendarContext.Provider, { value: calendar }, providerProps.children));
}
/**
 * Return the calendar state and navigation methods to navigate the calendar.
 *
 * Use this hook from the custom components passed via the `components` prop.
 *
 * @group Hooks
 * @see https://react-day-picker.js.org/advanced-guides/custom-components
 */
export function useCalendar() {
    const context = useContext(calendarContext);
    if (!context)
        throw new Error(`useCalendar must be used within a CalendarProvider.`);
    return context;
}
//# sourceMappingURL=calendar.js.map