import React, { createContext, useContext, useEffect, useState } from "react";
import { DayModifier } from "../UI";
import { getNextFocus } from "../helpers/getNextFocus";
import { useCalendar } from "./calendar";
import { useModifiers } from "./modifiers";
import { useProps } from "./props";
const focusContext = createContext(undefined);
/** @private */
export function FocusProvider({ children }) {
    const { goToDay, isDayDisplayed, days, isInteractive } = useCalendar();
    const { autoFocus = false, ...props } = useProps();
    const { calendarModifiers, getModifiers } = useModifiers();
    const [focused, setFocused] = useState();
    const [lastFocused, setLastFocused] = useState();
    const [initiallyFocused, setInitiallyFocused] = useState(false);
    const today = calendarModifiers.today[0];
    let autoFocusTarget;
    const isValidFocusTarget = (day) => {
        return isDayDisplayed(day) && !getModifiers(day)[DayModifier.disabled];
    };
    if (isInteractive) {
        if (focused) {
            autoFocusTarget = focused;
        }
        else if (lastFocused) {
            autoFocusTarget = lastFocused;
        }
        else if (calendarModifiers.selected[0] &&
            isValidFocusTarget(calendarModifiers.selected[0])) {
            autoFocusTarget = calendarModifiers.selected[0];
        }
        else if (today && isValidFocusTarget(today)) {
            autoFocusTarget = today;
        }
        else if (calendarModifiers.focusable[0]) {
            autoFocusTarget = calendarModifiers.focusable[0];
        }
    }
    // Focus the focus target when autoFocus is passed in
    useEffect(() => {
        if (!autoFocus)
            return;
        if (!autoFocusTarget)
            return;
        if (!initiallyFocused)
            return;
        setInitiallyFocused(true);
        setFocused(autoFocusTarget);
    }, [autoFocus, autoFocusTarget, focused, initiallyFocused]);
    function blur() {
        setLastFocused(focused);
        setFocused(undefined);
    }
    function moveFocus(moveBy, moveDir) {
        if (!focused)
            return;
        const nextFocus = getNextFocus(moveBy, moveDir, focused, props);
        if (!nextFocus)
            return;
        goToDay(nextFocus);
        setFocused(nextFocus);
    }
    const value = {
        autoFocusTarget,
        initiallyFocused,
        focusedDay: focused,
        blur,
        focus: setFocused,
        focusDayAfter: () => moveFocus("day", "after"),
        focusDayBefore: () => moveFocus("day", "before"),
        focusWeekAfter: () => moveFocus("week", "after"),
        focusWeekBefore: () => moveFocus("week", "before"),
        focusMonthBefore: () => moveFocus("month", "before"),
        focusMonthAfter: () => moveFocus("month", "after"),
        focusYearBefore: () => moveFocus("year", "before"),
        focusYearAfter: () => moveFocus("year", "after"),
        focusStartOfWeek: () => moveFocus("startOfWeek", "before"),
        focusEndOfWeek: () => moveFocus("endOfWeek", "after")
    };
    return (React.createElement(focusContext.Provider, { value: value }, children));
}
/**
 * Share the focused day and the methods to move the focus.
 *
 * Use this hook from the custom components passed via the `components` prop.
 *
 * @group Hooks
 * @see https://react-day-picker.js.org/advanced-guides/custom-components
 */
export function useFocus() {
    const context = useContext(focusContext);
    if (!context) {
        throw new Error("useFocus must be used within a FocusProvider");
    }
    return context;
}
//# sourceMappingURL=focus.js.map