import React, { createContext, useContext } from "react";
import { addDays } from "date-fns/addDays";
import { differenceInCalendarDays } from "date-fns/differenceInCalendarDays";
import { isSameDay } from "date-fns/isSameDay";
import { subDays } from "date-fns/subDays";
import { useControlledValue } from "../helpers/useControlledValue";
import { addToRange } from "../utils/addToRange";
import { isDateInRange } from "../utils/isDateInRange";
import { isDateRange } from "../utils/typeguards";
import { useProps } from "./props";
const contextValue = {
    selected: undefined,
    setSelected: () => undefined,
    isSelected: () => false,
    isStartOfRange: () => false,
    isEndOfRange: () => false,
    isMiddleOfRange: () => false
};
const selectionContext = createContext(contextValue);
/** @private */
export function SelectionProvider(providerProps) {
    const { required, min, max, onSelect, selected, defaultSelected, mode } = useProps();
    const [value, setValue] = useControlledValue(defaultSelected ?? selected, selected);
    /** Set the selected days when in "single" mode. */
    function setSingle(date, modifiers, e) {
        let selected;
        if (modifiers.selected && !required) {
            selected = undefined;
        }
        else {
            selected = date;
        }
        setValue(selected);
        onSelect?.(selected, date, modifiers, e);
        return selected;
    }
    /** Return `true` if the given day is selected in "single" mode. */
    function isSingleSelected(date) {
        return Boolean(value && isSameDay(value, date));
    }
    /** Set the selected days when in "multiple" mode. */
    function setMulti(date, modifiers, e) {
        if (value !== undefined && !Array.isArray(value)) {
            // Not a multi select
            return;
        }
        let selected = [];
        if (modifiers.selected) {
            // Date is already selected
            if (min && value && value.length <= min) {
                // Min value reached, do nothing
                selected = value;
            }
            else {
                // Remove the date from the selection
                selected = value?.filter((day) => !isSameDay(day, date)) || [];
            }
        }
        else if (max !== undefined && value?.length === max) {
            // Max value reached, reset the selection to date
            selected = [date];
        }
        else {
            // Add the date to the selection
            selected = [...(value ?? []), date];
        }
        setValue(selected);
        onSelect?.(selected, date, modifiers, e);
        return selected;
    }
    function isMultiSelected(date) {
        if (!Array.isArray(value))
            return false;
        return Boolean(value?.some((day) => isSameDay(day, date)));
    }
    /** Set the selected days when in "range" mode. */
    function setRange(date, modifiers, e) {
        if (value !== undefined && !isDateRange(value)) {
            return;
        }
        const selected = addToRange(date, value);
        if (min) {
            if (selected?.from &&
                selected.to &&
                differenceInCalendarDays(selected.to, selected.from) <= min) {
                selected.from = date;
                selected.to = undefined;
            }
        }
        if (max) {
            if (selected?.from &&
                selected.to &&
                differenceInCalendarDays(selected.to, selected.from) + 1 > max) {
                selected.from = date;
                selected.to = undefined;
            }
        }
        setValue(selected);
        onSelect?.(selected, date, modifiers, e); // Now TypeScript knows it's a MouseEvent
        return value;
    }
    function isRangeSelected(date) {
        if (value !== undefined && !isDateRange(value)) {
            // Not a range select
            return false;
        }
        return Boolean(value && isDateInRange(date, value));
    }
    function setSelected(date, modifiers, e) {
        if (mode === "single")
            setSingle(date, modifiers, e);
        if (mode === "multiple")
            setMulti(date, modifiers, e);
        if (mode === "range")
            setRange(date, modifiers, e);
    }
    function isSelected(date) {
        if (mode === "single")
            return isSingleSelected(date);
        if (mode === "multiple")
            return isMultiSelected(date);
        if (mode === "range")
            return isRangeSelected(date);
        return false;
    }
    function isStartOfRange(date) {
        if (!isDateRange(value))
            return false;
        return Boolean(value.from && isSameDay(value.from, date) && value.to);
    }
    function isEndOfRange(date) {
        if (!isDateRange(value))
            return false;
        return Boolean(value.to && isSameDay(value.to, date));
    }
    function isMiddleOfRange(date) {
        if (!isDateRange(value))
            return false;
        if (!value.from || !value.to)
            return false;
        return (isDateInRange(date, {
            from: addDays(value.from, 1),
            to: subDays(value.to, 1)
        }) &&
            !isStartOfRange(date) &&
            !isEndOfRange(date));
    }
    const contextValue = {
        selected: value,
        setSelected,
        isSelected,
        isStartOfRange,
        isEndOfRange,
        isMiddleOfRange
    };
    return (React.createElement(selectionContext.Provider, { value: contextValue }, providerProps.children));
}
/**
 * Access and change the currently selected values.
 *
 * Use this hook from the custom components passed via the `components` prop.
 *
 * @group Hooks
 * @see https://react-day-picker.js.org/advanced-guides/custom-components
 */
export function useSelection() {
    const context = useContext(selectionContext);
    if (!context) {
        throw new Error(`useSelection must be used within a SelectionProvider.`);
    }
    return context;
}
//# sourceMappingURL=selection.js.map